@@ -426,7 +426,7 @@ static void usage(const char *msg)
 					"         [--version]\n"
 					"         [--use-syslog]\n"
 					"         [--debug-<type> <level>]\n"
-					"           <type>:  log context type (dmn|mgr|ike|chd|job|cfg|knl|net|enc|tnc|imc|imv|pts|tls|lib)\n"
+					"           <type>:  log context type (dmn|mgr|ike|chd|job|cfg|knl|net|asn|enc|tnc|imc|imv|pts|tls|lib)\n"
 					"           <level>: log verbosity (-1 = silent, 0 = audit, 1 = control,\n"
 					"                                    2 = controlmore, 3 = raw, 4 = private)\n"
 					"\n"
@@ -497,6 +497,7 @@ int main(int argc, char *argv[])
 			{ "debug-cfg", required_argument, &group, DBG_CFG },
 			{ "debug-knl", required_argument, &group, DBG_KNL },
 			{ "debug-net", required_argument, &group, DBG_NET },
+			{ "debug-asn", required_argument, &group, DBG_ASN },
 			{ "debug-enc", required_argument, &group, DBG_ENC },
 			{ "debug-tnc", required_argument, &group, DBG_TNC },
 			{ "debug-imc", required_argument, &group, DBG_IMC },
