@@ -414,7 +414,6 @@ METHOD(stroke_list_t, status, void,
 	if (all)
 	{
 		peer_cfg_t *peer_cfg;
-		plugin_t *plugin;
 		char *pool;
 		host_t *host;
 		u_int32_t dpd;
@@ -449,14 +448,8 @@ METHOD(stroke_list_t, status, void,
 		}
 		fprintf(out, ", scheduled: %d\n",
 				lib->scheduler->get_job_load(lib->scheduler));
-		fprintf(out, "  loaded plugins: ");
-		enumerator = lib->plugins->create_plugin_enumerator(lib->plugins);
-		while (enumerator->enumerate(enumerator, &plugin, NULL))
-		{
-			fprintf(out, "%s ", plugin->get_name(plugin));
-		}
-		enumerator->destroy(enumerator);
-		fprintf(out, "\n");
+		fprintf(out, "  loaded plugins: %s\n",
+				lib->plugins->loaded_plugins(lib->plugins));
 
 		first = TRUE;
 		enumerator = this->attribute->create_pool_enumerator(this->attribute);
