@@ -559,6 +559,10 @@ METHOD(ike_sa_t, send_dpd, status_t,
 	job_t *job;
 	time_t diff, delay;
 
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
 	delay = this->peer_cfg->get_dpd(this->peer_cfg);
 	if (this->task_manager->busy(this->task_manager))
 	{
@@ -615,6 +619,8 @@ METHOD(ike_sa_t, get_state, ike_sa_state_t,
 METHOD(ike_sa_t, set_state, void,
 	private_ike_sa_t *this, ike_sa_state_t state)
 {
+	bool trigger_dpd = FALSE;
+
 	DBG2(DBG_IKE, "IKE_SA %s[%d] state change: %N => %N",
 		 get_name(this), this->unique_id,
 		 ike_sa_state_names, this->state,
@@ -675,12 +681,7 @@ METHOD(ike_sa_t, set_state, void,
 					lib->scheduler->schedule_job(lib->scheduler, job, t);
 					DBG1(DBG_IKE, "maximum IKE_SA lifetime %ds", t);
 				}
-
-				/* start DPD checks */
-				if (this->peer_cfg->get_dpd(this->peer_cfg))
-				{
-					send_dpd(this);
-				}
+				trigger_dpd = this->peer_cfg->get_dpd(this->peer_cfg);
 			}
 			break;
 		}
@@ -697,6 +698,11 @@ METHOD(ike_sa_t, set_state, void,
 	}
 	charon->bus->ike_state_change(charon->bus, &this->public, state);
 	this->state = state;
+
+	if (trigger_dpd)
+	{
+		send_dpd(this);
+	}
 }
 
 METHOD(ike_sa_t, reset, void,
@@ -1431,6 +1437,11 @@ METHOD(ike_sa_t, rekey_child_sa, status_t,
 {
 	child_rekey_t *child_rekey;
 
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
+
 	child_rekey = child_rekey_create(&this->public, protocol, spi);
 	this->task_manager->queue_task(this->task_manager, &child_rekey->task);
 	return this->task_manager->initiate(this->task_manager);
@@ -1441,6 +1452,11 @@ METHOD(ike_sa_t, delete_child_sa, status_t,
 {
 	child_delete_t *child_delete;
 
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
+
 	child_delete = child_delete_create(&this->public, protocol, spi);
 	this->task_manager->queue_task(this->task_manager, &child_delete->task);
 	return this->task_manager->initiate(this->task_manager);
@@ -1500,6 +1516,10 @@ METHOD(ike_sa_t, rekey, status_t,
 {
 	ike_rekey_t *ike_rekey;
 
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
 	ike_rekey = ike_rekey_create(&this->public, TRUE);
 
 	this->task_manager->queue_task(this->task_manager, &ike_rekey->task);
@@ -1511,6 +1531,10 @@ METHOD(ike_sa_t, reauth, status_t,
 {
 	task_t *task;
 
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
 	/* we can't reauthenticate as responder when we use EAP or virtual IPs.
 	 * If the peer does not support RFC4478, there is no way to keep the
 	 * IKE_SA up. */
@@ -1708,6 +1732,10 @@ static void requeue_init_tasks(private_ike_sa_t *this)
 METHOD(ike_sa_t, retransmit, status_t,
 	private_ike_sa_t *this, u_int32_t message_id)
 {
+	if (this->state == IKE_PASSIVE)
+	{
+		return INVALID_STATE;
+	}
 	this->stats[STAT_OUTBOUND] = time_monotonic(NULL);
 	if (this->task_manager->retransmit(this->task_manager, message_id) != SUCCESS)
 	{
