@@ -37,7 +37,7 @@ static const char imv_name[] = "Test";
 static imv_agent_t *imv_test;
 
 /**
- * see section 3.7.1 of TCG TNC IF-IMV Specification 1.2
+ * see section 3.8.1 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_Initialize(TNC_IMVID imv_id,
 							  TNC_Version min_version,
@@ -64,16 +64,13 @@ TNC_Result TNC_IMV_Initialize(TNC_IMVID imv_id,
 }
 
 /**
- * see section 3.7.2 of TCG TNC IF-IMV Specification 1.2
+ * see section 3.8.2 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_NotifyConnectionChange(TNC_IMVID imv_id,
 										  TNC_ConnectionID connection_id,
 										  TNC_ConnectionState new_state)
 {
 	imv_state_t *state;
-	imv_test_state_t *test_state;
-	TNC_Result result;
-	int rounds;
 
 	if (!imv_test)
 	{
@@ -87,60 +84,29 @@ TNC_Result TNC_IMV_NotifyConnectionChange(TNC_IMVID imv_id,
 			return imv_test->create_state(imv_test, state);
 		case TNC_CONNECTION_STATE_DELETE:
 			return imv_test->delete_state(imv_test, connection_id);
-		case TNC_CONNECTION_STATE_HANDSHAKE:
-			/* get updated IMV state */
-			result = imv_test->change_state(imv_test, connection_id,
-											new_state, &state);
-			if (result != TNC_RESULT_SUCCESS)
-			{
-				return result;
-			}
-			test_state = (imv_test_state_t*)state;
-
-			/* set the number of measurement rounds */
-			rounds = lib->settings->get_int(lib->settings,
-								"libimcv.plugins.imv-test.rounds", 0);
-			test_state->set_rounds(test_state, rounds);
-			return TNC_RESULT_SUCCESS;
 		default:
 			return imv_test->change_state(imv_test, connection_id,
 										  new_state, NULL);
 	}
 }
 
-static TNC_Result send_message(TNC_ConnectionID connection_id)
-{
-	pa_tnc_msg_t *msg;
-	pa_tnc_attr_t *attr;
-	TNC_Result result;
-
-	attr = ita_attr_command_create("repeat");
-	msg = pa_tnc_msg_create();
-	msg->add_attribute(msg, attr);
-	msg->build(msg);
-	result = imv_test->send_message(imv_test, connection_id,
-									msg->get_encoding(msg));	
-	msg->destroy(msg);
-
-	return result;
-}
-
-/**
- * see section 3.7.3 of TCG TNC IF-IMV Specification 1.2
- */
-TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imv_id,
+static TNC_Result receive_message(TNC_IMVID imv_id,
 								  TNC_ConnectionID connection_id,
-								  TNC_BufferReference msg,
-								  TNC_UInt32 msg_len,
-								  TNC_MessageType msg_type)
+								  TNC_UInt32 msg_flags,
+								  chunk_t msg,
+								  TNC_VendorID msg_vid,
+								  TNC_MessageSubtype msg_subtype,
+								  TNC_UInt32 src_imc_id,
+								  TNC_UInt32 dst_imv_id)
 {
 	pa_tnc_msg_t *pa_tnc_msg;
 	pa_tnc_attr_t *attr;
 	imv_state_t *state;
-	imv_test_state_t *imv_test_state;
+	imv_test_state_t *test_state;
 	enumerator_t *enumerator;
 	TNC_Result result;
-	bool fatal_error = FALSE, retry = FALSE;
+	int rounds;
+	bool fatal_error, retry = FALSE;
 
 	if (!imv_test)
 	{
@@ -153,11 +119,11 @@ TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imv_id,
 	{
 		return TNC_RESULT_FATAL;
 	}
+	test_state = (imv_test_state_t*)state;
 
 	/* parse received PA-TNC message and automatically handle any errors */ 
-	result = imv_test->receive_message(imv_test, connection_id,
-									   chunk_create(msg, msg_len), msg_type,
-									   &pa_tnc_msg);
+	result = imv_test->receive_message(imv_test, state, msg, msg_vid,
+					 		msg_subtype, src_imc_id, dst_imv_id, &pa_tnc_msg);
 
 	/* no parsed PA-TNC attributes available if an error occurred */
 	if (!pa_tnc_msg)
@@ -165,41 +131,20 @@ TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imv_id,
 		return result;
 	}
 
+	/* preprocess any IETF standard error attributes */
+	fatal_error = pa_tnc_msg->process_ietf_std_errors(pa_tnc_msg);
+
+	/* add any new IMC and set its number of rounds */
+	rounds = lib->settings->get_int(lib->settings,
+								"libimcv.plugins.imv-test.rounds", 0);
+	test_state->add_imc(test_state, src_imc_id, rounds);
+
 	/* analyze PA-TNC attributes */
 	enumerator = pa_tnc_msg->create_attribute_enumerator(pa_tnc_msg);
 	while (enumerator->enumerate(enumerator, &attr))
 	{
-		if (attr->get_vendor_id(attr) == PEN_IETF &&
-			attr->get_type(attr) == IETF_ATTR_PA_TNC_ERROR)
-		{
-			ietf_attr_pa_tnc_error_t *error_attr;
-			pa_tnc_error_code_t error_code;
-			chunk_t msg_info, attr_info;
-			u_int32_t offset;
-
-			error_attr = (ietf_attr_pa_tnc_error_t*)attr;
-			error_code = error_attr->get_error_code(error_attr);
-			msg_info = error_attr->get_msg_info(error_attr);
-
-			DBG1(DBG_IMV, "received PA-TNC error '%N' concerning message %#B",
-				 pa_tnc_error_code_names, error_code, &msg_info);
-			switch (error_code)
-			{
-				case PA_ERROR_INVALID_PARAMETER:
-					offset = error_attr->get_offset(error_attr);
-					DBG1(DBG_IMV, "  occurred at offset of %u bytes", offset);
-					break;
-				case PA_ERROR_ATTR_TYPE_NOT_SUPPORTED:
-					attr_info = error_attr->get_attr_info(error_attr);
-					DBG1(DBG_IMV, "  unsupported attribute %#B", &attr_info);
-					break;
-				default:
-					break;
-			}
-			fatal_error = TRUE;
-		}
-		else if (attr->get_vendor_id(attr) == PEN_ITA &&
-				 attr->get_type(attr) == ITA_ATTR_COMMAND)
+		if (attr->get_vendor_id(attr) == PEN_ITA &&
+			attr->get_type(attr) == ITA_ATTR_COMMAND)
 		{
 			ita_attr_command_t *ita_attr;
 			char *command;
@@ -252,22 +197,67 @@ TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imv_id,
 	/* request a handshake retry ? */
 	if (retry)
 	{
+		test_state->set_rounds(test_state, rounds);
 		return imv_test->request_handshake_retry(imv_id, connection_id,
 								TNC_RETRY_REASON_IMV_SERIOUS_EVENT);
 	}
 	
 	/* repeat the measurement ? */
-	imv_test_state = (imv_test_state_t*)state;
-	if (imv_test_state->another_round(imv_test_state))
+	if (test_state->another_round(test_state, src_imc_id))
 	{
-		return send_message(connection_id);
+		attr = ita_attr_command_create("repeat");
+		pa_tnc_msg = pa_tnc_msg_create();
+		pa_tnc_msg->add_attribute(pa_tnc_msg, attr);
+		pa_tnc_msg->build(pa_tnc_msg);
+		result = imv_test->send_message(imv_test, connection_id, TRUE, imv_id,
+							src_imc_id, pa_tnc_msg->get_encoding(pa_tnc_msg));	
+		pa_tnc_msg->destroy(pa_tnc_msg);
+
+		return result;
 	}
 
 	return imv_test->provide_recommendation(imv_test, connection_id);
 }
 
 /**
- * see section 3.7.4 of TCG TNC IF-IMV Specification 1.2
+ * see section 3.8.4 of TCG TNC IF-IMV Specification 1.3
+ */
+TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imv_id,
+								  TNC_ConnectionID connection_id,
+								  TNC_BufferReference msg,
+								  TNC_UInt32 msg_len,
+								  TNC_MessageType msg_type)
+{
+	TNC_VendorID msg_vid;
+	TNC_MessageSubtype msg_subtype;
+
+	msg_vid = msg_type >> 8;
+	msg_subtype = msg_type & TNC_SUBTYPE_ANY;
+
+	return receive_message(imv_id, connection_id, 0, chunk_create(msg, msg_len),
+						   msg_vid,	msg_subtype, 0, TNC_IMVID_ANY);
+}
+
+/**
+ * see section 3.8.6 of TCG TNC IF-IMV Specification 1.3
+ */
+TNC_Result TNC_IMV_ReceiveMessageLong(TNC_IMVID imv_id,
+									  TNC_ConnectionID connection_id,
+									  TNC_UInt32 msg_flags,
+									  TNC_BufferReference msg,
+									  TNC_UInt32 msg_len,
+									  TNC_VendorID msg_vid,
+									  TNC_MessageSubtype msg_subtype,
+									  TNC_UInt32 src_imc_id,
+									  TNC_UInt32 dst_imv_id)
+{
+	return receive_message(imv_id, connection_id, msg_flags,
+						   chunk_create(msg, msg_len), msg_vid, msg_subtype,
+						   src_imc_id, dst_imv_id);
+}
+
+/**
+ * see section 3.8.7 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_SolicitRecommendation(TNC_IMVID imv_id,
 										 TNC_ConnectionID connection_id)
@@ -281,7 +271,7 @@ TNC_Result TNC_IMV_SolicitRecommendation(TNC_IMVID imv_id,
 }
 
 /**
- * see section 3.7.5 of TCG TNC IF-IMV Specification 1.2
+ * see section 3.8.8 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_BatchEnding(TNC_IMVID imv_id,
 							   TNC_ConnectionID connection_id)
@@ -295,7 +285,7 @@ TNC_Result TNC_IMV_BatchEnding(TNC_IMVID imv_id,
 }
 
 /**
- * see section 3.7.6 of TCG TNC IF-IMV Specification 1.2
+ * see section 3.8.9 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_Terminate(TNC_IMVID imv_id)
 {
@@ -311,7 +301,7 @@ TNC_Result TNC_IMV_Terminate(TNC_IMVID imv_id)
 }
 
 /**
- * see section 4.2.8.1 of TCG TNC IF-IMV Specification 1.2
+ * see section 4.2.8.1 of TCG TNC IF-IMV Specification 1.3
  */
 TNC_Result TNC_IMV_ProvideBindFunction(TNC_IMVID imv_id,
 									   TNC_TNCS_BindFunctionPointer bind_function)
