@@ -25,6 +25,8 @@
 #ifndef LIBPTS_H_
 #define LIBPTS_H_
 
+#include "pts/components/pts_component_manager.h"
+
 #include <library.h>
 
 /**
@@ -39,4 +41,9 @@ bool libpts_init(void);
  */
 void libpts_deinit(void);
 
+/**
+ * PTS Functional Component manager
+ */
+extern pts_component_manager_t* pts_components;
+
 #endif /** LIBPTS_H_ @}*/
