@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tobias Brunner
+ * Copyright (C) 2010-2012 Tobias Brunner
  * Copyright (C) 2007 Martin Willi
  * Hochschule fuer Technik Rapperswil
  *
@@ -45,6 +45,11 @@ struct private_plugin_loader_t {
 	 * List of plugins, as plugin_entry_t
 	 */
 	linked_list_t *plugins;
+
+	/**
+	 * List of names of loaded plugins
+	 */
+	char *loaded_plugins;
 };
 
 /**
@@ -203,6 +208,38 @@ METHOD(plugin_loader_t, create_plugin_enumerator, enumerator_t*,
 							(void*)plugin_filter, NULL, NULL);
 }
 
+/**
+ * Create a list of the names of all loaded plugins
+ */
+static char* loaded_plugins_list(private_plugin_loader_t *this)
+{
+	int buf_len = 128, len = 0;
+	char *buf, *name;
+	enumerator_t *enumerator;
+	plugin_t *plugin;
+
+	buf = malloc(buf_len);
+	buf[0] = '\0';
+	enumerator = create_plugin_enumerator(this);
+	while (enumerator->enumerate(enumerator, &plugin, NULL))
+	{
+		name = plugin->get_name(plugin);
+		if (len + (strlen(name) + 1) >= buf_len)
+		{
+			buf_len <<= 1;
+			buf = realloc(buf, buf_len);
+		}
+		len += snprintf(&buf[len], buf_len - len, "%s ", name);
+	}
+	enumerator->destroy(enumerator);
+	if (len > 0 && buf[len - 1] == ' ')
+	{
+		buf[len - 1] = '\0';
+	}
+	return buf;
+}
+
+
 /**
  * Check if a plugin is already loaded
  */
@@ -292,7 +329,7 @@ static bool dependencies_satisfied(private_plugin_loader_t *this,
 				name = entry->plugin->get_name(entry->plugin);
 				provide = plugin_feature_get_string(&features[0]);
 				depend = plugin_feature_get_string(&features[i]);
-				DBG1(DBG_LIB, "feature %s in '%s' plugin has unsatisfied "
+				DBG2(DBG_LIB, "feature %s in '%s' plugin has unsatisfied "
 					 "dependency: %s", provide, name, depend);
 				free(provide);
 				free(depend);
@@ -516,6 +553,11 @@ METHOD(plugin_loader_t, load_plugins, bool,
 		/* unload plugins that we were not able to load any features for */
 		purge_plugins(this);
 	}
+	if (!critical_failed)
+	{
+		free(this->loaded_plugins);
+		this->loaded_plugins = loaded_plugins_list(this);
+	}
 	return !critical_failed;
 }
 
@@ -558,6 +600,8 @@ METHOD(plugin_loader_t, unload, void,
 		}
 		enumerator->destroy(enumerator);
 	}
+	free(this->loaded_plugins);
+	this->loaded_plugins = NULL;
 }
 
 /**
@@ -606,11 +650,18 @@ METHOD(plugin_loader_t, reload, u_int,
 	return reloaded;
 }
 
+METHOD(plugin_loader_t, loaded_plugins, char*,
+	private_plugin_loader_t *this)
+{
+	return this->loaded_plugins ?: "";
+}
+
 METHOD(plugin_loader_t, destroy, void,
 	private_plugin_loader_t *this)
 {
 	unload(this);
 	this->plugins->destroy(this->plugins);
+	free(this->loaded_plugins);
 	free(this);
 }
 
@@ -627,6 +678,7 @@ plugin_loader_t *plugin_loader_create()
 			.reload = _reload,
 			.unload = _unload,
 			.create_plugin_enumerator = _create_plugin_enumerator,
+			.loaded_plugins = _loaded_plugins,
 			.destroy = _destroy,
 		},
 		.plugins = linked_list_create(),
