@@ -42,6 +42,13 @@ struct leak_detective_t {
 	 */
 	void (*report)(leak_detective_t *this, bool detailed);
 
+	/**
+	 * Report current memory usage to out.
+	 *
+	 * @param out			target to write usage report to
+	 */
+	void (*usage)(leak_detective_t *this, FILE *out);
+
 	/**
 	 * Destroy a leak_detective instance.
 	 */
