@@ -264,26 +264,6 @@ static const char *pkcs11_init_args = NULL;
 /* options read by optionsfrom */
 options_t *options;
 
-/**
- * Log loaded plugins
- */
-static void print_plugins()
-{
-	char buf[BUF_LEN];
-	plugin_t *plugin;
-	int len = 0;
-	enumerator_t *enumerator;
-
-	buf[0] = '\0';
-	enumerator = lib->plugins->create_plugin_enumerator(lib->plugins);
-	while (len < BUF_LEN && enumerator->enumerate(enumerator, &plugin, NULL))
-	{
-		len += snprintf(&buf[len], BUF_LEN-len, "%s ", plugin->get_name(plugin));
-	}
-	enumerator->destroy(enumerator);
-	DBG1(DBG_DMN, "loaded plugins: %s", buf);
-}
-
 int main(int argc, char **argv)
 {
 	bool fork_desired = TRUE;
@@ -675,6 +655,9 @@ int main(int argc, char **argv)
 		close(fd);
 	}
 
+	/* for uncritical pseudo random numbers */
+	srand(time(NULL) + getpid());
+
 	init_constants();
 	init_log("pluto");
 
@@ -698,7 +681,8 @@ int main(int argc, char **argv)
 	{
 		exit(SS_RC_INITIALIZATION_FAILED);
 	}
-	print_plugins();
+	DBG1(DBG_DMN, "loaded plugins: %s",
+		 lib->plugins->loaded_plugins(lib->plugins));
 
 	init_builder();
 	if (!init_secret() || !init_crypto())
@@ -852,6 +836,7 @@ void exit_pluto(int status)
 	delete_lock();
 	options->destroy(options);
 	pluto_deinit();
+	lib->credmgr->flush_cache(lib->credmgr, CERT_ANY);
 	lib->plugins->unload(lib->plugins);
 	libhydra_deinit();
 	library_deinit();
